@@ -2,7 +2,6 @@ import asyncio
 import pyppeteer
 import random
 import time
-import json
 import string
 import atexit
 import requests
@@ -12,7 +11,7 @@ import logging
 from .stealth import stealth
 
 class browser:
-    def __init__(self, url, language='en', proxy=None, find_redirect=False, api_url=None, debug=False):
+    def __init__(self, url, language='en', proxy=None, find_redirect=False, api_url=None, debug=False, newParams=False):
         self.url = url
         self.debug = debug
         self.proxy = proxy
@@ -20,7 +19,7 @@ class browser:
         self.referrer = "https://www.tiktok.com/"
         self.language = language
 
-        self.userAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.0 Safari/537.36)"
+        self.userAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.125 Safari/537.36"
         self.args = [
             "--no-sandbox",
             "--disable-setuid-sandbox",
@@ -50,12 +49,32 @@ class browser:
             'handleSIGHUP': False
         }
 
-        loop = asyncio.new_event_loop()
+        try:
+            self.loop = asyncio.new_event_loop()
+            if find_redirect:
+                self.loop.run_until_complete(self.find_redirect())
+            elif newParams:
+                self.loop.run_until_complete(self.newParams())
+            else:
+                self.loop.run_until_complete(self.start())
+        except:
+            self.loop.close()
 
-        if find_redirect:
-            loop.run_until_complete(self.find_redirect())
-        else:
-            loop.run_until_complete(self.start())
+    async def newParams(self):
+        self.browser = await pyppeteer.launch(self.options)
+        self.page = await self.browser.newPage()
+        await self.page.goto("about:blank")
+
+        self.browser_language = await self.page.evaluate("""() => { return navigator.language || navigator.userLanguage; }""")
+        self.timezone_name = await self.page.evaluate("""() => { return Intl.DateTimeFormat().resolvedOptions().timeZone; }""")
+        self.browser_platform = await self.page.evaluate("""() => { return window.navigator.platform; }""")
+        self.browser_name = await self.page.evaluate("""() => { return window.navigator.appCodeName; }""")
+        self.browser_version = await self.page.evaluate("""() => { return window.navigator.appVersion; }""")
+        self.width = await self.page.evaluate("""() => { return screen.width; }""")
+        self.height = await self.page.evaluate("""() => { return screen.height; }""")
+
+        await self.browser.close()
+        self.browser.process.communicate()
 
     async def start(self):
         self.browser = await pyppeteer.launch(self.options)
@@ -75,13 +94,6 @@ class browser:
 
         await stealth(self.page)
 
-        # await self.page.emulate({
-        #    'viewport': {'width': random.randint(320, 1920), 'height': random.randint(320, 1920), },
-        #    'deviceScaleFactor': random.randint(1, 3),
-        #    'isMobile': random.random() > 0.5,
-        #    'hasTouch': random.random() > 0.5
-        # })
-
         # might have to switch to a tiktok url if they improve security
         await self.page.goto("about:blank", {
             'waitUntil': "load"
@@ -92,7 +104,7 @@ class browser:
         self.verifyFp = ''.join(random.choice(string.ascii_lowercase + string.ascii_uppercase + string.digits) for i in range(16))
 
         await self.page.evaluate("() => { " + self.__get_js(proxy=self.proxy) + " }")
-        
+
         self.signature = await self.page.evaluate('''() => {
         var url = "''' + self.url + "&verifyFp=" + self.verifyFp + '''"
         var token = window.byted_acrawler.sign({url: url});
@@ -107,10 +119,8 @@ class browser:
                                 })
 
             self.data = await self.page.content()
-            print(self.data)
-            #self.data = await json.loads(self.data)
+            self.data = json.loads(self.data.replace("</pre></body></html>", "").replace('<html><head></head><body><pre style="word-wrap: break-word; white-space: pre-wrap;">', ""))
 
-        await self.browser.close()
         await self.browser.close()
         self.browser.process.communicate()
 
@@ -166,4 +176,4 @@ class browser:
             return None
 
     def __get_js(self, proxy=None):
-        return requests.get("https://sf16-muse-va.ibytedtos.com/obj/rc-web-sdk-gcs/acrawler.js", proxies=self.__format_proxy(proxy)).text
\ No newline at end of file
+        return requests.get("https://sf16-muse-va.ibytedtos.com/obj/rc-web-sdk-gcs/acrawler.js", proxies=self.__format_proxy(proxy)).text
